/* -*- C++ -*- *************************************************************
 *
 *  lsf_minuit.cpp, Mon Aug 23 10:09:21 MSD 2004
 *  Copyright (C) 2004 Soloviev Alexei <solovjev@cv.jinr.ru>, Alexei Stadnik
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lsf.h"

/** Static pointer to this for FCN */
const ChiSquare * ChiSquare::__lsf_ptr = 0;

const string ChiSquare::__strategies[] = {
  "MINIMIZE",
  "MIGRAD",
  "SCAN",
  "SEEK",
  "SIMPLEX"
};

/** User FCN function for MINUIT */
void ChiSquare::fcn ( int & npar, double *, double & chi2, double * par, int )
{
  if ( par ) {
    for ( register int i = 0; i < npar; ++i ) {
      (*__lsf_ptr->__m_ptr)[i] = par[i];
    }
    chi2 = __lsf_ptr->value();
  }
}

/** Set printout level for MINUIT */
void ChiSquare::set_mn_printout ( const int level )
{
  static int ierflg;
  /** SET PRIntout <level>
   *  Sets the print level, determining how much output will be
   *  produced. Allowed values and their meanings are displayed
   *  after a SHOw PRInt command, and are currently <level>=:
   *    [-1]  no output except from SHOW commands
   *     [0]  minimum output
   *     [1]  default value, normal output
   *     [2]  additional output giving intermediate results
   *     [3]  maximum output, showing progress of minimizations
   */
  switch ( level ) {
  case -1: mncomd ( "SET PRINTOUT -1", ierflg ); break;
  case 0:  mncomd ( "SET PRINTOUT 0",  ierflg ); break;
  case 1:  mncomd ( "SET PRINTOUT 1",  ierflg ); break;
  case 2:  mncomd ( "SET PRINTOUT 2",  ierflg ); break;
  case 3:  mncomd ( "SET PRINTOUT 3",  ierflg ); break;
  default: mncomd ( "SET PRINTOUT 3",  ierflg ); break;
  }
  mncomd ( "SET NOWARNINGS", ierflg );
}

/** Set parameters for MINUIT */
void ChiSquare::set_mn_parms()
{
  static int ierflg;
  for( int i = 0; i < model().size(); ++i ) {
    mnparm ( i,
	     model().name(i).c_str(),
	     model()[i],
	     model().error(i),
	     model().min(i),
	     model().max(i),
	     ierflg );
  }
}

/** Get parameters from MINUIT */
void ChiSquare::get_mn_parms() const
{
  for( int i = 0; i < model().size(); ++i ) {
    GetParameter ( i, (*__m_ptr)[i], __m_ptr->error(i) );
  }
}

