/***************************************************************************
                          gintegral.h  -  description
                             -------------------
    begin                : Tue Apr 23 2002
    copyright            : (C) 2002 by Alexei Soloviev, Alexander Polanski
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GINTEGRAL_H
#define GINTEGRAL_H

#include<cmath>

/**Gaussian quadrature
  *@author Alexei Soloviev, Alexander Polanski
  */

template<class function>
double integral ( const function & f,
                  const double & a,
                  const double & b,
                  const double & error = 0.1 );

/***************************************************************************
 *                                                                         *
 *   Implementation                                                        *
 *                                                                         *
 ***************************************************************************/

class GIntegral {
 private:
  const int order;
  const double *x, *w;
 protected:
  GIntegral ( const int n, const double * xx, const double * ww );
 public:
  ~GIntegral();

  static const GIntegral g4();
  static const GIntegral g8();

  template<class function>
    inline const double operator () ( const function &f,
				      const double &a,
				      const double &b ) const
    {
      const double c1 = (a + b) / 2, c2 = c1 - a;
      double res = 0;
      for(int i = 0; i < order; ++i)
	res += w[i] * (f(c1 + c2 * x[i]) + f(c1 - c2 * x[i]));
      return res *= c2;
    }
};

template<class function>
double integral ( const function & f,
                  const double & a,
                  const double & b,
                  const double & error )
{
  GIntegral i4 = GIntegral::g4();
  GIntegral i8 = GIntegral::g8();

  const double e = 1e-12, delta = e * fabs(a - b);

  double res = 0, s4, s8;
  for ( double aa = a, bb, y = b - a;
	fabs(y) > delta;
	aa = bb, y = b - aa ) {
    do {
      bb = aa + y;
      s4 = i4 ( f, aa, bb );
      s8 = i8 ( f, aa, bb );
      y /= 2;
    } while ( fabs ( s8 - s4 ) > error * ( 1 + fabs ( s8 ) ) );
    res += s8;
  }
  return res;
}

#endif
