/* -*- C++ -*- *************************************************************
 *
 *  model.cpp, Tue Mar 5 2002
 *  Copyright (C) 2002 Soloviev Alexei <solovjev@cv.jinr.ru>, Alexei Stadnik
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "model.h"

#include <QSettings>

Model::Model(const vector<Parameter> & rhs, const string & name)
  : fParm(rhs), fName(name)
{}

Model::~Model() {}

/** Number of parameters */
size_type Model::size() const 
{
  return fParm.size();
}

/** Access to parameter no. index (read) */
const double & Model::operator[](size_type index) const 
{
  return fParm[index].value();
}

/** Access to parameter no. index (write) */
double & Model::operator[](size_type index) 
{
  return fParm[index].value();
}

/** Access to error of parameter no. index (read) */
const double & Model::error(size_type index) const
{
  return fParm[index].error();
}

/** Access to min value of parameter no. index (read) */
const double & Model::min(size_type index) const
{
  return fParm[index].min();
}

/** Access to max value of parameter no. index (read) */
const double & Model::max(size_type index) const
{
  return fParm[index].max();
}

/** Access to error of parameter no. index (write) */
double & Model::error(size_type index) 
{
  return fParm[index].error();
}

/** Name of parameter no. index */
const string & Model::name(size_type index) const 
{
  return fParm[index].name();
}

/** Model name */
const string & Model::name() const 
{
  return fName;
}

/** Read model parameters from rc-file */
void Model::rc_read()
{
    QSettings settings(QSettings::IniFormat, QSettings::UserScope, "YuMO", PACKAGE);
    settings.setIniCodec("UTF-8");

    settings.sync();

    QStringList groups = settings.childGroups();
    if ( groups.contains ( name().c_str() ) ) {
        settings.beginGroup ( name().c_str() );
        for ( int i = 0; i < size(); ++i ) {
            QList<QVariant> par = settings.value ( name(i).c_str() ).toList();
            fParm[i].value() = par.first().toDouble();
            fParm[i].error() = par.last().toDouble();
        }
        settings.endGroup();
    }
}

/** Write model parameters to rc-file */
void Model::rc_write() const
{
    QSettings settings(QSettings::IniFormat, QSettings::UserScope, "YuMO", PACKAGE);
    settings.setIniCodec("UTF-8");

    settings.beginGroup ( name().c_str() );
    for ( int i = 0; i < size(); ++i ) {
        QList<QVariant> par = {fParm[i].value(), fParm[i].error()};
        settings.setValue ( name(i).c_str(), par );
    }
    settings.endGroup();

    settings.sync();
}
