/***************************************************************************
                          stdmodel.h  -  description
                             -------------------
    begin                : Fri Apr 19 2002
    copyright            : (C) 2002 by Alexei Soloviev, Alexei Stadnik
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef STDMODEL_H
#define STDMODEL_H

#include "model.h"

#include <map>
#include <string>

/**Standart models
  *@author Alexei Soloviev, Alexei Stadnik
  */

class STDModel : public Model  {
 private: // Private attributes
  typedef double (STDModel::*TF) ( const double & ) const;

 protected: // Protected constructor
  STDModel ( const std::vector<Parameter> & p,
	     const TF & f, const std::string & n );

 public:
  virtual ~STDModel ();

 public:
  /** Theoretical function */
  virtual double operator () ( const double & x ) const;

 public:
  /** To construct model over comand line parameters */
  static const Dictionary<STDModel> & dictionary();

 public:
  /** Gaussian constructor */
  static const STDModel gaussian ();
  /** Exponential constructor */
  static const STDModel exponential ();
  /** Polynomial constructor */
  static const STDModel polynomial ( const unsigned int order );
  /** Polynomial constructors for users */
  static const STDModel polynomial0 ();
  static const STDModel polynomial1 ();
  static const STDModel polynomial2 ();
  static const STDModel polynomial3 ();
  static const STDModel polynomial4 ();
  static const STDModel polynomial5 ();
  static const STDModel polynomial6 ();
  static const STDModel polynomial7 ();
  static const STDModel polynomial8 ();

 private: // Private attributes
  /** Pointer to theoretical function for current model */
  const TF fTheorFuncPtr;

 private: // Private methods
  /** Gaussian theoretical function */
  double gaussian ( const double & x ) const;
  /** Exponential theoretical function */
  double exponential ( const double & x ) const;
  /** Polynomial theoretical function */
  double polynomial ( const double & x ) const;
};

#endif
